a + b
a - b
a * b
a / b
a // b
a % b
a @ b
a << b
a >> b
a | b
a ^ b
a ** b
a == b
a < b
a <= b
a > b
a >= b
a != b
a & b
~a
(1, 2, 3)
["a", "b"]
{1, 2}
{a: a.b}
{**d, a: b}

not b
a if b else c
a or b
a and b
a in b
a not in b
a is b
a is not b

a * (+1)
a * (-1)
a * (~1)

(a)
(yield a)