print("import: 'pegen'")
import pegen

print("import: 'pegen.grammar'")
import pegen.grammar

print("import: 'pegen.grammar_parser'")
import pegen.grammar_parser

print("import: 'pegen.grammar_visualizer'")
import pegen.grammar_visualizer

print("import: 'pegen.parser'")
import pegen.parser

print("import: 'pegen.parser_generator'")
import pegen.parser_generator

print("import: 'pegen.python_generator'")
import pegen.python_generator

print("import: 'pegen.tokenizer'")
import pegen.tokenizer

print("import: 'pegen.utils'")
import pegen.utils

