async def f():
    pass


async def f():
    await b


async def f():
    async for i in range(10):
        pass


async def f():
    async with open(f) as p:
        pass


async def f():
    a = [i async for i in range(10)]
    return a
